@@ -159,11 +159,11 @@ print_tmap <- function(x, vp=NULL, return.asp=FALSE, mode=getOption("tmap.mode")
 	} else which(is_master)[1]
 	
 	## find master projection
-	master_proj <- get_proj4(x[[shape.id[masterID]]]$projection)
+	master_CRS <- get_proj4(x[[shape.id[masterID]]]$projection, as.CRS = TRUE)
 	mshp_raw <- x[[shape.id[masterID]]]$shp
-	if (is.null(master_proj)) master_proj <- get_projection(mshp_raw)
-	orig_proj <- master_proj # needed for adjusting bbox in process_shapes
-	if (interactive) master_proj <- get_proj4("longlat")
+	if (is.null(master_CRS)) master_CRS <- get_projection(mshp_raw, as.CRS = TRUE)
+	orig_CRS <- master_CRS # needed for adjusting bbox in process_shapes
+	if (interactive) master_CRS <- .CRS_longlat
 	
 	## find master bounding box (unprocessed)
 	bbx_raw <- bb(mshp_raw)
@@ -183,7 +183,7 @@ print_tmap <- function(x, vp=NULL, return.asp=FALSE, mode=getOption("tmap.mode")
 	})
 	
 	## split data.frames from shape/raster objects, and determine shape types
-	shps_dts <- mapply(preprocess_shapes, x[shape.id], raster_facets_vars, MoreArgs = list(apply_map_coloring=apply_map_coloring, master_proj=master_proj, master_bbx=bbx_raw, interactive=interactive), SIMPLIFY = FALSE)
+	shps_dts <- mapply(preprocess_shapes, x[shape.id], raster_facets_vars, MoreArgs = list(apply_map_coloring=apply_map_coloring, master_CRS=master_CRS, master_bbx=bbx_raw, interactive=interactive), SIMPLIFY = FALSE)
 	
 	shps <- lapply(shps_dts, "[[", 1)
 	datasets <- lapply(shps_dts, "[[", 2)
@@ -264,7 +264,7 @@ print_tmap <- function(x, vp=NULL, return.asp=FALSE, mode=getOption("tmap.mode")
 	# aspect ratio per facet minus extern legend
 	lasp <- fasp * (1-fpi$legmarx) / (1-fpi$legmary)
 	
-	shps <- process_shapes(shps, x[shape.id], gmeta, data_by, lasp, masterID, allow.crop = !interactive, raster.leaflet=interactive, projection=master_proj, interactive=interactive, orig.projection=orig_proj)
+	shps <- process_shapes(shps, x[shape.id], gmeta, data_by, lasp, masterID, allow.crop = !interactive, raster.leaflet=interactive, master_CRS=master_CRS, interactive=interactive, orig_CRS=orig_CRS)
 	
 	sasp <- attr(shps, "sasp")
 	bbx <- attr(shps, "bbx")
