@@ -206,6 +206,8 @@ process_meta <- function(gt, gf, gg, gc, gl, gsb, gcomp, glab, nx, panel.names,
 			grid.col <- do.call("process_color", c(list(col=grid.col, alpha=grid.alpha), gt$pc))
 			grid.labels.col <- do.call("process_color", c(list(col=grid.labels.col), gt$pc))
 			grid.lwd <- grid.lwd * gt$scale
+			grid.is.projected <- !grid.projection=="longlat"
+			grid.projection <- get_proj4(grid.projection, as.CRS = TRUE)
 		})
 	} else {
 		gg <- list(grid.show=FALSE)
