@@ -12,68 +12,62 @@
 #' 
 #' @param shp shape object of class \code{\link[sp:Spatial]{Spatial}} or
 #'   \code{\link[raster:Raster-class]{Raster}} (see details).
-#' @param projection character that determines the new projection. Either a \code{PROJ.4} character string or a shortcut. See \code{\link{get_proj4}} for a list of shortcut values. This argument is
-#'   only used to transform the \code{shp}. Use \code{current.projection} to
-#'   specify the current projection of \code{shp}.
-#' @param current.projection the current projection of \code{shp}. Only use this
-#'   if the current projection is missing or wrong.
-#' @param overwrite.current.projection logical that determines whether the
-#'   current projection is overwritten if it already has a projection that is
-#'   different.
+#' @param projection new projection. Either a \code{\link[sp:CRS]{CRS}} object or a character value. If it is a character, it can either be a \code{PROJ.4} character string or a shortcut. See \code{\link{get_proj4}} for a list of shortcut values. This argument is only used to transform the \code{shp}. Use \code{current.projection} to specify the current projection of \code{shp}.
+#' @param current.projection the current projection of \code{shp}. See \code{projection} for possible formats. Only use this if the current projection is missing or wrong.
+#' @param overwrite.current.projection logical that determines whether the current projection is overwritten if it already has a projection that is different.
+#' @param as.CRS should a CRS object be returned instead of a PROJ.4 character string? Default is \code{FALSE}.
 #' @name set_projection
 #' @rdname set_projection
 #' @import sp
 #' @importFrom raster projectRaster
 #' @importFrom rgdal getPROJ4VersionInfo
-#' @return \code{set_projection} returns a (transformed) shape object with
-#'   updated projection information. \code{get_projection} returns the
-#'   \code{PROJ.4} character string of \code{shp}.
+#' @return \code{set_projection} returns a (transformed) shape object with updated projection information. \code{get_projection} returns the \code{PROJ.4} character string of \code{shp}.
 #' @export
-set_projection <- function(shp, projection=NULL, current.projection=NULL, overwrite.current.projection=FALSE) {
+set_projection <- function(shp, projection=NA, current.projection=NA, overwrite.current.projection=FALSE) {
 	shp.name <- deparse(substitute(shp))
-	shp.proj <- proj4string(shp)
+	
+	shp.CRS <- get_projection(shp, as.CRS = TRUE)
+	current.CRS <- get_proj4(current.projection, as.CRS=TRUE)
+	proj.CRS <- get_proj4(projection, as.CRS=TRUE)
 
-	current.proj4 <- get_proj4(current.projection)
-
-	if (is.na(shp.proj)) {
-		if (missing(current.projection) || is.na(current.projection)) {
+	if (is.na(shp.CRS)) {
+		if (is.na(current.CRS)) {
 			stop("Currect projection of shape object unknown. Please specify the argument current.projection. The value \"longlat\", which stands for Longitude-latitude (WGS84), is most commonly used.")
 		} else {
 			if (inherits(shp, "Spatial")) {
-				shp@proj4string <- CRS(current.proj4)
+				shp@proj4string <- current.CRS
 			} else {
-				shp@crs <- CRS(current.proj4) 
+				shp@crs <- current.CRS
 			}
-			current.projection <- current.proj4
+			#current.projection <- current.proj4
 		}
 	} else {
-		if (!missing(current.projection)) {
-			if (current.proj4==shp.proj) {
+		if (!is.na(current.CRS)) {
+			if (identical(current.CRS, shp.CRS)) {
 				warning("Current projection of ", shp.name, " already known.", call. = FALSE)
 			} else {
 				if (overwrite.current.projection) {
-					warning("Current projection of ", shp.name, " differs from ", current.projection, ", but is overwritten.", call. = FALSE)
+					warning("Current projection of ", shp.name, " differs from ", CRSargs(current.CRS), ", but is overwritten.", call. = FALSE)
 					if (inherits(shp, "Spatial")) {
-						shp@proj4string <- CRS(current.proj4)
+						shp@proj4string <- current.CRS
 					} else {
-						shp@crs <- CRS(current.proj4) 
+						shp@crs <- current.CRS
 					}
 					
 				} else {
-					stop(shp.name, " already has projection: ", shp.proj, ". This is different from the specified current projection ", current.projection, ". If the specified projection is correct, use overwrite.current.projection=TRUE.", call. = FALSE)
+					stop(shp.name, " already has projection: ", CRSargs(shp.CRS), ". This is different from the specified current projection ", CRSargs(current.CRS), ". If the specified projection is correct, use overwrite.current.projection=TRUE.", call. = FALSE)
 				}
 			} 
 		} else {
-			current.proj4 <- shp.proj
+			current.CRS <- shp.CRS
 		}
 	}
 	
 
-	if (!missing(projection)) {
-		proj4 <- get_proj4(projection)
+	if (!is.na(proj.CRS)) {
 		PROJ4_version_nr <- get_proj4_version()
 		
-		if (length(grep("+proj=wintri", current.proj4, fixed = TRUE)) && PROJ4_version_nr < 491) {
+		if (length(grep("+proj=wintri", CRSargs(current.CRS), fixed = TRUE)) && PROJ4_version_nr < 491) {
 			stop("Unable to reproject a shape from the Winkel Tripel projection with PROJ.4 version < 4.9.1")
 		}	
 		
@@ -98,12 +92,12 @@ set_projection <- function(shp, projection=NULL, current.projection=NULL, overwr
 #				raster_data <- data.frame(PIXEL__COLOR=getValues(shp[[1]])+1L)
 			}
 			isnum <- sapply(lvls, is.null)
-			new_ext <- suppressWarnings(projectExtent(shp, crs = CRS(proj4)))
+			new_ext <- suppressWarnings(projectExtent(shp, crs = proj.CRS))
 			if (any(isnum) && !all(isnum)) {
 				shp_num <- raster::subset(shp, subset=which(isnum))
 				shp_cat <- raster::subset(shp, subset=which(!isnum))
-				shp_num2 <- suppressWarnings(projectRaster(shp_num, to=new_ext, crs=proj4, method="bilinear"))
-				shp_cat2 <- suppressWarnings(projectRaster(shp_cat, to=new_ext, crs=proj4, method="ngb"))
+				shp_num2 <- suppressWarnings(projectRaster(shp_num, to=new_ext, crs=proj.CRS, method="bilinear"))
+				shp_cat2 <- suppressWarnings(projectRaster(shp_cat, to=new_ext, crs=proj.CRS, method="ngb"))
 				
 				# restore order
 				o <- order(c(which(isnum), which(!isnum)))
@@ -111,9 +105,9 @@ set_projection <- function(shp, projection=NULL, current.projection=NULL, overwr
 							 lapply(1:nlayers(shp_cat), function(i) raster(shp_cat2, layer=i)))[o]
 				shp <- do.call("brick", rlayers)
 			} else if (all(isnum)) {
-				shp <- suppressWarnings(projectRaster(shp, to=new_ext, crs=proj4, method="bilinear"))
+				shp <- suppressWarnings(projectRaster(shp, to=new_ext, crs=proj.CRS, method="bilinear"))
 			} else {
-				shp <- suppressWarnings(projectRaster(shp, to=new_ext, crs=proj4, method="ngb"))
+				shp <- suppressWarnings(projectRaster(shp, to=new_ext, crs=proj.CRS, method="ngb"))
 			}
 			
 			# new_raster_data <- as.data.frame(mapply(function(d, l) {
@@ -130,7 +124,13 @@ set_projection <- function(shp, projection=NULL, current.projection=NULL, overwr
 				shp@data@attributes <- dfs
 			}
 		} else {
-			shp <- spTransform(shp, CRS(proj4))
+			shp <- tryCatch({
+				spTransform(shp, proj.CRS)
+			}, error=function(e) {
+				stop("Unable to set the projection to ", CRSargs(proj.CRS), ".", call.=FALSE)
+			}, warning=function(w){
+				NULL
+			})
 		}
 		if (recast) {
 			shp <- as(shp, cls)
@@ -152,9 +152,17 @@ get_proj4_version <- function() {
 #' @rdname set_projection
 #' @import sp
 #' @export
-get_projection <- function(shp) {
-	proj4string(shp)
+get_projection <- function(shp, as.CRS=FALSE) {
+	if (as.CRS) {
+		if (inherits(shp, "Spatial")) {
+			attr(shp, "proj4string")
+		} else if (inherits(shp, "Raster")) {
+			attr(shp, "crs")
+		} else {
+			stop("shp is neither a Spatial nor a Raster object")
+		}
+	} else {
+		proj4string(shp)
+	}
 }
 
-
-
